@@ -1,4 +1,4 @@
-// Copyright (C) 1999-2001 Open Source Telecom Corporation.
+// Copyright (C) 1999-2005 Open Source Telecom Corporation.
 //  
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -13,30 +13,28 @@
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software 
 // Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
-// 
-// As a special exception to the GNU General Public License, permission is 
-// granted for additional uses of the text contained in its release
-// of ccaudio.
 //
-// The exception is that, if you link the ccaudio library with other
-// files to produce an executable, this does not by itself cause the
-// resulting executable to be covered by the GNU General Public License.
-// Your use of that executable is in no way restricted on account of
-// linking the ccaudio library code into it.
+// As a special exception, you may use this file as part of a free software
+// library without restriction.  Specifically, if other files instantiate
+// templates or use macros or inline functions from this file, or you compile
+// this file and link it with other files to produce an executable, this
+// file does not by itself cause the resulting executable to be covered by
+// the GNU General Public License.  This exception does not however    
+// invalidate any other reasons why the executable file might be covered by
+// the GNU General Public License.    
 //
-// This exception does not however invalidate any other reasons why
-// the executable file might be covered by the GNU General Public License.
-// 
-// This exception applies only to the code released under the 
-// name ccaudio.  If you copy code from other releases into a copy of
-// ccaudio, as the General Public License permits, the exception does
+// This exception applies only to the code released under the name GNU
+// ccAudio.  If you copy code from other releases into a copy of GNU
+// ccAudio, as the General Public License permits, the exception does
 // not apply to the code that you add in this way.  To avoid misleading
 // anyone as to the status of such modified files, you must delete
 // this exception notice from them.
-// 
-// If you write modifications of your own for ccaudio, it is your choice
+//
+// If you write modifications of your own for GNU ccAudio, it is your choice
 // whether to permit this exception to apply to your modifications.
-// If you do not wish that, delete this exception notice.  
+// If you do not wish that, delete this exception notice.
+//
+
 
 #ifndef	CCXX_AUDIO_H_
 #define	CCXX_AUDIO_H_
@@ -157,9 +155,12 @@ public:
 	enum	Mode
 	{
 		modeRead,
+		modeReadAny,
+		modeReadOne,
 		modeWrite,
 		modeCache,
-		modeInfo
+		modeInfo,
+		modeFeed
 	};
 
 	typedef enum Mode Mode;
