@@ -1,4 +1,4 @@
-// Copyright (C) 1999-2001 Open Source Telecom Corporation.
+// Copyright (C) 1999-2005 Open Source Telecom Corporation.
 //  
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -14,29 +14,26 @@
 // along with this program; if not, write to the Free Software 
 // Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 // 
-// As a special exception to the GNU General Public License, permission is 
-// granted for additional uses of the text contained in its release 
-// of ccaudio.
-// 
-// The exception is that, if you link the ccaudio library with other
-// files to produce an executable, this does not by itself cause the
-// resulting executable to be covered by the GNU General Public License.
-// Your use of that executable is in no way restricted on account of
-// linking the ccaudio library code into it.
-// 
-// This exception does not however invalidate any other reasons why
-// the executable file might be covered by the GNU General Public License.
-// 
-// This exception applies only to the code released under the 
-// name ccaudio.  If you copy code from other releases into a copy of
-// ccaudio, as the General Public License permits, the exception does
+// As a special exception, you may use this file as part of a free software
+// library without restriction.  Specifically, if other files instantiate
+// templates or use macros or inline functions from this file, or you compile
+// this file and link it with other files to produce an executable, this
+// file does not by itself cause the resulting executable to be covered by
+// the GNU General Public License.  This exception does not however
+// invalidate any other reasons why the executable file might be covered by
+// the GNU General Public License.
+//
+// This exception applies only to the code released under the name GNU
+// ccAudio.  If you copy code from other releases into a copy of GNU
+// ccAudio, as the General Public License permits, the exception does
 // not apply to the code that you add in this way.  To avoid misleading
 // anyone as to the status of such modified files, you must delete
 // this exception notice from them.
-// 
-// If you write modifications of your own for ccaudio, it is your choice
+//
+// If you write modifications of your own for GNU ccAudio, it is your choice
 // whether to permit this exception to apply to your modifications.
-// If you do not wish that, delete this exception notice.  
+// If you do not wish that, delete this exception notice.
+//
 
 #include "private.h"
 #include "audio2.h"
@@ -660,8 +657,16 @@ void AudioFile::open(const char *name, Mode m, timeout_t framing)
 	mode = m;
 
 
-	if(!afOpen(name, m))
-		return;
+	while(!afOpen(name, m))
+	{
+		if(mode == modeReadAny || mode == modeReadOne)
+			name = getContinuation();
+		else
+			name = NULL;
+
+		if(!name)
+			return;
+	}
 
 	pathname = new char[strlen(name) + 1];
 	strcpy(pathname, name);
@@ -847,6 +852,13 @@ done:
 		info.setFraming(framing);
 	else
 		info.set();
+
+	if(mode == modeFeed)
+	{
+		setPosition();
+		iolimit = toBytes(info, getPosition());
+		setPosition(0);
+	}
 }
 
 Audio::Info::Info()
@@ -1083,6 +1095,11 @@ int AudioFile::getBuffer(void *addr, unsigned bytes)
 	int count, curpos, xfer = 0;
 
         curpos = toBytes(info, getPosition());
+	if(curpos >= iolimit && mode == modeFeed)
+	{
+		curpos = 0;
+		setPosition(0);
+	}
         if (iolimit && ((curpos + bytes) > iolimit))
                 bytes = iolimit - curpos;
         if (bytes < 0)
@@ -1102,18 +1119,35 @@ int AudioFile::getBuffer(void *addr, unsigned bytes)
 		xfer += count;
 		if(count == (int)bytes)
 			return xfer;
-		fname = getContinuation();
+		if(mode == modeFeed)
+		{
+			setPosition(0l);
+			goto cont;
+		}
+
+retry:
+		if(mode == modeReadOne)
+			fname = NULL;
+		else
+			fname = getContinuation();
+
 		if(!fname)
 			return xfer;
 		AudioFile::close();
 		AudioFile::open(fname, modeRead, info.framing);
 		if(!isOpen())
+		{
+			if(mode == modeReadAny)
+				goto retry;
 			return xfer;
+		}
+
 		if(prior.encoding != info.encoding)
 		{
 			AudioFile::close();
 			return xfer;
 		}
+cont:
 		bytes -= count;
 		caddr += count;
 	}
@@ -1126,7 +1160,7 @@ Audio::Error AudioFile::getSamples(void *addr, unsigned request)
 	int count;
 	for(;;)
 	{
-		int bytes = toBytes(info.encoding, request);
+		int bytes = toBytes(info, request);
 		if(bytes < 1)
 			return setError(errRequestInvalid);
 		count = afRead(caddr, bytes);
@@ -1144,14 +1178,30 @@ Audio::Error AudioFile::getSamples(void *addr, unsigned request)
 		else
 			count = request;
 
-		fname = getContinuation();
+		if(mode == modeFeed)
+		{
+			setPosition(0);
+			request = count;
+			continue;
+		}
+
+retry:
+		if(mode == modeReadOne)
+			fname = NULL;
+		else
+			fname = getContinuation();
+
 		if(!fname)
 			break;
 
 		AudioFile::close();
 		AudioFile::open(fname, modeRead);
 		if(!isOpen())
+		{
+			if(mode == modeReadAny)
+				goto retry;
 			break;
+		}
 
 		request = count;
 	}
@@ -1163,6 +1213,20 @@ Audio::Error AudioFile::getSamples(void *addr, unsigned request)
 int AudioFile::putBuffer(void *addr, unsigned len)
 {
 	int count;
+	unsigned long curpos;
+
+	curpos = toBytes(info, getPosition());
+        if(curpos >= iolimit && mode == modeFeed)
+        {
+                curpos = 0;
+                setPosition(0);
+        }
+
+        if (iolimit && ((curpos + len) > iolimit))
+                len = iolimit - curpos;
+
+        if (len <= 0)
+		return 0;  
 
 	count = afWrite((unsigned char *)addr, len);
 	if(count == (int)len) {
